/*
	Fudroyd
	A Gameboy emulator for Android
	Mic, 2009
*/

#include "stdafx.h"
#include "gbz80.h"
#include "gui.h"
#include "memory.h"
#include "ppu.h"


int				run,quit,romLoaded;
char			szBuffer[256];
unsigned long	templ;
int				debg=0;


int load_rom(char *fname) {
	FILE *fp;
	int size;

	fp = fopen(fname,"rb");
	fseek(fp,0,SEEK_END);
	size = ftell(fp);
	fseek(fp,0,SEEK_SET);
	cart = (unsigned char*)malloc(size);
	fread(cart,1,size,fp);
	fclose(fp);

	numBanks = size/0x4000;

	printf("ROM type: %d, ROM size: %d bytes\n",cart[0x147],size);

	return 1;
}


void idle() {
	while ((!run) && (!quit)) {
		gui_process();

		if (!quit)
			SDL_Delay(20);
 	}
}


void reset_machine() {
	mem_reset();
	cpu_reset();
	ppu_reset();
}

void emulate() {
	while (run) {
		cpu.cycles = 0;
		ppu_mode(0);
		cpu_execute(204);
		ppu_mode(2);
		cpu_execute(285);
		ppu_mode(3);
		cpu_execute(456);
		if (ppu.scanline==0)
			gui_present(ppu.vscreen);
		gui_process();
	}
}


void shut_down() {
	if (romLoaded)
		free(cart);
	gui_close();
	cpu_close();
}


int main(int argc, char **argv) {
	int i;

	if (argc < 2) {
		fprintf(stderr, "Usage: fudroyd <filename.gb>\n");
		return 0;
	}

	gui_init();

	romLoaded = load_rom(argv[1]);
	reset_machine();

	SDL_WM_SetCaption("fudroyd",0);

	quit = 0;
	run = 1;
	romLoaded = 0;

	while (!quit) {
		idle();
		emulate();
	}
	shut_down();

	return 0;
}




