/*
	memory.cpp
	fudroyd memory access functions
	/mic 2009
*/

#include "memory.h"
#include "gbz80.h"

extern void cpu_rst(unsigned short);

unsigned char	RAM[0x2000],VRAM[0x2000],OAM[0xA0],IOREGS[0x4C],HIRAM[0x80],EXRAM[128*1024],MBC2RAM[512];
unsigned char	*ROM0,*ROM1,*RAM1;
unsigned char	mbc1Layout;
unsigned short	highRomBits,highRomBitsLatch;
unsigned char	IME,REG_IE,romSelect,ramSelect,ramSelectLatch,keys,numBanks,divcnt;
int				tmcnt,tmrld;


int mem_reset() {
	keys = 0xff;
	IME = 0;
	romSelect = 1;
	ramSelect = 0;
	mbc1Layout = 0;
	highRomBits = 0;
	divcnt = 0;
	REG_DIV = 0;
	REG_TAC = 0;
	ROM0 = cart;
	ROM1 = ROM0+0x4000;
	RAM1 = NULL;
	return 1;
}



unsigned char mem_read_8(unsigned short address) {
	if (address<0x4000) {
		return ROM0[address];

	} else if (address<0x8000) {
		if (ROM1 != NULL)
			return ROM1[address-0x4000];

	} else if (address<0xA000) {
		return VRAM[address-0x8000];

	} else if (address<0xC000) {
		if (RAM1 != NULL)
			return RAM1[address-0xA000];
		else if ((address<0xA200)&&((CART_TYPE==TYPE_MBC2)||(CART_TYPE==TYPE_MBC2_BATT)))
		     return MBC2RAM[address-0xA000]&0xF;

	} else if (address < 0xFE00) {
		if (address==0xCC06) {
			sprintf(szBuffer,"Reading %02X from %04X\n",RAM[address&0x1FFF],address);
			//WriteConsole(hout,szBuffer,strlen(szBuffer),&templ,NULL);
		}
		return RAM[address&0x1FFF];

	} else if (address < 0xFEA0) {
		return OAM[address-0xFE00];

	} else if ((address>=0xFF00)&&(address<0xFF4C)) {
		if (address==0xFF00) {
			//debg = 1;
			//sprintf(szBuffer,"Reading from %04X\n",address);
			if ((IOREGS[0]&0x30)==0x10)
				return (IOREGS[0]&0xF0)|((keys>>4)&0xF);
			else if ((IOREGS[0]&0x30)==0x20)
				return (IOREGS[0]&0xF0)|(keys&0xF);
			else
				return IOREGS[0];
		} else
		{
		if (address==0xFF0F) {
			//sprintf(szBuffer,"Reading %02X from %04X\n",REG_IF,address);
		}

 			return IOREGS[address-0xFF00];
        }

	} else if ((address>=0xFF80)&&(address<0xFFFF)) {
		return HIRAM[address-0xFF80];

	} else if (address==0xFFFF) {
		return REG_IE;
	}

	return 0;
}


unsigned short mem_read_16(unsigned short address) {
	return mem_read_8(address) + ((unsigned short)mem_read_8(address+1)<<8);
}


void mem_write_8(unsigned short address,unsigned char data) {
	int i;

	if (address<0x4000) {
		if ((CART_TYPE==TYPE_MBC1)||(CART_TYPE==TYPE_MBC1_RAM)||(CART_TYPE==TYPE_MBC1_RAM_BATT)) {
			if (address<0x2000) {
               if ((data&0x0F)==0x0A)
                  RAM1 = &EXRAM[ramSelect*0x2000];
               else
                  RAM1 = NULL;
            } else if (address>=0x2000) {
				romSelect = (data==0)?1:(data&0x1F);
				romSelect &= (numBanks-1);
	   	        //sprintf(szBuffer,"MBC1: Selecting bank %d\n",romSelect+highRomBits);
                //WriteConsole(hout,szBuffer,strlen(szBuffer),&templ,NULL);
				if (mbc1Layout==0)
					ROM1 = &cart[(romSelect+highRomBits)*0x4000];
				else
					ROM1 = &cart[(romSelect)*0x4000];
			}
	//	} else if (CART_TYPE==3) {
		/*	if (address>=0x2000) {
		sprintf(szBuffer,"Writing %02X to %04X\n",data,address);
		//WriteConsole(hout,szBuffer,strlen(szBuffer),&templ,NULL);
				romSelect = (data==0)?1:(data&0x7F);
				romSelect &= (numBanks-1);
				ROM1 = &cart[romSelect*0x4000];
			}*/
		}
		else if ((CART_TYPE==TYPE_MBC2)||(CART_TYPE==TYPE_MBC2_BATT))
		{
             romSelect = (data==0)?1:(data&0xF);
             ROM1 = &cart[(romSelect)*0x4000];
        }

	} else if (address<0x8000) {
		if ((CART_TYPE==TYPE_MBC1)||(CART_TYPE==TYPE_MBC1_RAM)||(CART_TYPE==TYPE_MBC1_RAM_BATT)) {
			if (address<0x6000) {
				if (mbc1Layout==0)
					highRomBitsLatch = highRomBits = (data&3)<<5;
				else {
					ramSelectLatch = ramSelect = data&3;
					//RAM1 = &EXRAM[ramSelect*0x2000];
				}
			} else {
				mbc1Layout = data&1;
				if (mbc1Layout==0)
					{ ramSelect = 0; highRomBits = highRomBitsLatch; } //RAM1 = NULL;
				else
					{ highRomBits = 0; ramSelect = ramSelectLatch; }//RAM1 = &EXRAM[ramSelect*0x2000];
			}
			RAM1 = &EXRAM[ramSelect*0x2000];
			ROM1 = &cart[(romSelect+highRomBits)*0x4000];
		}

	} else if (address<0xA000) {
		VRAM[address-0x8000] = data;

	} else if (address<0xC000) {
		if (RAM1 != NULL)
			RAM1[address-0xA000] = data;
		else if ((address<0xA200)&&((CART_TYPE==TYPE_MBC2)||(CART_TYPE==TYPE_MBC2_BATT)))
		    MBC2RAM[address-0xA000] = data&0xF;

	} else if (address < 0xFE00) {
		RAM[address&0x1FFF] = data;
		/*if (address==0xC014) {
			sprintf(szBuffer,"Writing %02X to %04X at %04X\n",data,address,cpu.regs.PC);
			WriteConsole(hout,szBuffer,strlen(szBuffer),&templ,NULL);
			debg=0;
		}*/

	} else if (address < 0xFEA0) {
		OAM[address-0xFE00] = data;
		/*if ((address&3)==0) {
			debg = 0;
			sprintf(szBuffer,"Writing %02X to %04X\n",data,address);
			WriteConsole(hout,szBuffer,strlen(szBuffer),&templ,NULL);
		}*/

	} else if ((address>=0xFF00)&&(address<0xFF4C)) {
		IOREGS[address-0xFF00] = data;
		if (address==0xFF00) { //((address==0xFF0F)||(address==0xFF40)||(address==0xFF41)||(address==0xFF44)) {
			sprintf(szBuffer,"Writing %02X to %04X at %04X\n",data,address,cpu.regs.PC);
			//WriteConsole(hout,szBuffer,strlen(szBuffer),&templ,NULL);
		}
		if (address==0xFF04) {
			IOREGS[0xFF04] = 0;
		} else if (address==0xFF07) {
			sprintf(szBuffer,"Writing %02X to %04X\n",data,address);
			//WriteConsole(hout,szBuffer,strlen(szBuffer),&templ,NULL);
			//if (data&4) while (1) {}
			switch (data&3) {
				case 0:
					tmrld = 1024;
					break;
				case 1:
					tmrld = 16;
					break;
				case 2:
					tmrld = 64;
					break;
				case 3:
					tmrld = 256;
					break;
			}
			tmcnt = 0;
		} else if (address==0xFF46) {
			for (i=0; i<40*4; i++)
				*(OAM+i) = mem_read_8(data*0x100 + i);
		}

	} else if ((address>=0xFF80)&&(address<0xFFFF)) {
		HIRAM[address-0xFF80] = data;

	} else if (address==0xFFFF) {
		REG_IE = data;
	}
}


void mem_write_16(unsigned short address,unsigned short data) {
	mem_write_8(address,data&0xFF);
	mem_write_8(address+1,data>>8);
}


void mem_key_down() {
	if ((keys&2)==0)
		debg = 40;
	cpu.stopped = 0;
/*	REG_IF |= 0x10;
	if ((IME!=0)&&((REG_IE&0x10)!=0)) {
		REG_IF &= ~0x10;
		cpu_rst(0x60);
		IME = 0;
	}*/
}
